@@ -12,6 +12,7 @@
 #include <stdlib.h>
 
 #include <iostream>
+#include <stdexcept>
 
 namespace fasttext {
 
@@ -35,7 +36,7 @@ Args::Args() {
   label = "__label__";
   verbose = 2;
   pretrainedVectors = "";
-  saveOutput = 0;
+  saveOutput = false;
 
   qout = false;
   retrain = false;
@@ -56,6 +57,14 @@ std::string Args::lossToString(loss_name ln) {
   return "Unknown loss!"; // should never happen
 }
 
+std::string Args::boolToString(bool b) {
+  if (b) {
+    return "true";
+  } else {
+    return "false";
+  }
+}
+
 void Args::parseArgs(const std::vector<std::string>& args) {
   std::string command(args[1]);
   if (command == "supervised") {
@@ -68,87 +77,93 @@ void Args::parseArgs(const std::vector<std::string>& args) {
   } else if (command == "cbow") {
     model = model_name::cbow;
   }
-  int ai = 2;
-  while (ai < args.size()) {
+  for (int ai = 2; ai < args.size(); ai += 2) {
     if (args[ai][0] != '-') {
       std::cerr << "Provided argument without a dash! Usage:" << std::endl;
       printHelp();
       exit(EXIT_FAILURE);
     }
-    if (args[ai] == "-h") {
-      std::cerr << "Here is the help! Usage:" << std::endl;
-      printHelp();
-      exit(EXIT_FAILURE);
-    } else if (args[ai] == "-input") {
-      input = std::string(args[ai + 1]);
-    } else if (args[ai] == "-test") {
-      test = std::string(args[ai + 1]);
-    } else if (args[ai] == "-output") {
-      output = std::string(args[ai + 1]);
-    } else if (args[ai] == "-lr") {
-      lr = std::stof(args[ai + 1]);
-    } else if (args[ai] == "-lrUpdateRate") {
-      lrUpdateRate = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-dim") {
-      dim = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-ws") {
-      ws = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-epoch") {
-      epoch = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-minCount") {
-      minCount = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-minCountLabel") {
-      minCountLabel = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-neg") {
-      neg = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-wordNgrams") {
-      wordNgrams = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-loss") {
-      if (args[ai + 1] == "hs") {
-        loss = loss_name::hs;
-      } else if (args[ai + 1] == "ns") {
-        loss = loss_name::ns;
-      } else if (args[ai + 1] == "softmax") {
-        loss = loss_name::softmax;
+    try {
+      if (args[ai] == "-h") {
+        std::cerr << "Here is the help! Usage:" << std::endl;
+        printHelp();
+        exit(EXIT_FAILURE);
+      } else if (args[ai] == "-input") {
+        input = std::string(args.at(ai + 1));
+      } else if (args[ai] == "-output") {
+        output = std::string(args.at(ai + 1));
+      } else if (args[ai] == "-lr") {
+        lr = std::stof(args.at(ai + 1));
+      } else if (args[ai] == "-lrUpdateRate") {
+        lrUpdateRate = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-dim") {
+        dim = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-ws") {
+        ws = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-epoch") {
+        epoch = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-minCount") {
+        minCount = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-minCountLabel") {
+        minCountLabel = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-neg") {
+        neg = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-wordNgrams") {
+        wordNgrams = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-loss") {
+        if (args.at(ai + 1) == "hs") {
+          loss = loss_name::hs;
+        } else if (args.at(ai + 1) == "ns") {
+          loss = loss_name::ns;
+        } else if (args.at(ai + 1) == "softmax") {
+          loss = loss_name::softmax;
+        } else {
+          std::cerr << "Unknown loss: " << args.at(ai + 1) << std::endl;
+          printHelp();
+          exit(EXIT_FAILURE);
+        }
+      } else if (args[ai] == "-bucket") {
+        bucket = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-minn") {
+        minn = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-maxn") {
+        maxn = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-thread") {
+        thread = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-t") {
+        t = std::stof(args.at(ai + 1));
+      } else if (args[ai] == "-label") {
+        label = std::string(args.at(ai + 1));
+      } else if (args[ai] == "-verbose") {
+        verbose = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-pretrainedVectors") {
+        pretrainedVectors = std::string(args.at(ai + 1));
+      } else if (args[ai] == "-saveOutput") {
+        saveOutput = true;
+        ai--;
+      } else if (args[ai] == "-qnorm") {
+        qnorm = true;
+        ai--;
+      } else if (args[ai] == "-retrain") {
+        retrain = true;
+        ai--;
+      } else if (args[ai] == "-qout") {
+        qout = true;
+        ai--;
+      } else if (args[ai] == "-cutoff") {
+        cutoff = std::stoi(args.at(ai + 1));
+      } else if (args[ai] == "-dsub") {
+        dsub = std::stoi(args.at(ai + 1));
       } else {
-        std::cerr << "Unknown loss: " << args[ai + 1] << std::endl;
+        std::cerr << "Unknown argument: " << args[ai] << std::endl;
         printHelp();
         exit(EXIT_FAILURE);
       }
-    } else if (args[ai] == "-bucket") {
-      bucket = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-minn") {
-      minn = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-maxn") {
-      maxn = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-thread") {
-      thread = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-t") {
-      t = std::stof(args[ai + 1]);
-    } else if (args[ai] == "-label") {
-      label = std::string(args[ai + 1]);
-    } else if (args[ai] == "-verbose") {
-      verbose = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-pretrainedVectors") {
-      pretrainedVectors = std::string(args[ai + 1]);
-    } else if (args[ai] == "-saveOutput") {
-      saveOutput = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-qnorm") {
-      qnorm = true; ai--;
-    } else if (args[ai] == "-retrain") {
-      retrain = true; ai--;
-    } else if (args[ai] == "-qout") {
-      qout = true; ai--;
-    } else if (args[ai] == "-cutoff") {
-    cutoff = std::stoi(args[ai + 1]);
-    } else if (args[ai] == "-dsub") {
-      dsub = std::stoi(args[ai + 1]);
-    } else {
-      std::cerr << "Unknown argument: " << args[ai] << std::endl;
+    } catch (std::out_of_range) {
+      std::cerr << args[ai] << " is missing an argument" << std::endl;
       printHelp();
       exit(EXIT_FAILURE);
     }
-    ai += 2;
   }
   if (input.empty() || output.empty()) {
     std::cerr << "Empty input or output path." << std::endl;
@@ -202,16 +217,16 @@ void Args::printTrainingHelp() {
     << "  -loss               loss function {ns, hs, softmax} [" << lossToString(loss) << "]\n"
     << "  -thread             number of threads [" << thread << "]\n"
     << "  -pretrainedVectors  pretrained word vectors for supervised learning ["<< pretrainedVectors <<"]\n"
-    << "  -saveOutput         whether output params should be saved [" << saveOutput << "]\n";
+    << "  -saveOutput         whether output params should be saved [" << boolToString(saveOutput) << "]\n";
 }
 
 void Args::printQuantizationHelp() {
   std::cerr
     << "\nThe following arguments for quantization are optional:\n"
     << "  -cutoff             number of words and ngrams to retain [" << cutoff << "]\n"
-    << "  -retrain            finetune embeddings if a cutoff is applied [" << retrain << "]\n"
-    << "  -qnorm              quantizing the norm separately [" << qnorm << "]\n"
-    << "  -qout               quantizing the classifier [" << qout << "]\n"
+    << "  -retrain            whether embeddings are finetuned if a cutoff is applied [" << boolToString(retrain) << "]\n"
+    << "  -qnorm              whether the norm is quantized separately [" << boolToString(qnorm) << "]\n"
+    << "  -qout               whether the classifier is quantized [" << boolToString(qout) << "]\n"
     << "  -dsub               size of each sub-vector [" << dsub << "]\n";
 }
 
